import{ world, system, MinecraftEffectTypes, MinecraftBlockTypes  } from "@minecraft/server";

let playerHurt = [];
let rollCooldown = [];
let rollTime = [];
let staminaDuration = [];
let stamina_is_cooldown = [];
let player_on_ground = [];
let roll_cooldown_actionbar = [
	"§8ROLL",
	"§eR§8OLL",
	"§eRO§8LL",
	"§eROL§8L",
	"§eROLL",
	"ROLL"
];
let stamina_cooldown_actionbar = [
	"STAMINA",
	"§eSTAMINA",
	"§eSTAMIN§8A",
	"§eSTAMI§8NA",
	"§eSTAM§8INA",
	"§eSTA§8MINA",
	"§eST§8AMINA",
	"§eS§8TAMINA",
	"§8STAMINA"
];

system.runInterval(() => {
	let creative_player = world.getPlayers({gameMode: "creative"});
	for(let playerData of world.getPlayers()){
		let is_creative = false;
		for(let player of creative_player){
			if(player.id == playerData.id){
				is_creative = true;
				break;
			}
		}
		
		let is_riding = false;
		for(let ride of playerData.dimension.getEntities({ location: playerData.location, excludeTypes: ["minecraft:player"], maxDistance: 1})){
			if(ride.hasComponent("minecraft:rideable")){
				is_riding = true;
				break;
			}
		}
		
		let vel = playerData.getVelocity();
		let is_in_liquid = playerData.dimension.getBlock(playerData.location).isLiquid();
		let player_speed = playerData.getComponent("minecraft:movement");
		let vel_calc = Math.sqrt(Math.pow(vel.x, 2) + Math.pow(vel.z, 2));
		let is_sprint = (vel_calc - player_speed.current*2) > 0.0175;
		if(!player_on_ground[playerData.id] && Math.abs(vel.y) < 0.005){
			player_on_ground[playerData.id] = true;
		}
		if(!is_creative){
			if(player_on_ground[playerData.id] && vel.y > 0.25 && !playerData.getEffect(MinecraftEffectTypes.jumpBoost) && !is_riding){
				player_on_ground[playerData.id] = false;
				staminaDuration[playerData.id] += 10;
			}
			
			if(staminaDuration[playerData.id] == undefined) staminaDuration[playerData.id] = 0;
			if(is_sprint && !is_riding){
				if(staminaDuration[playerData.id] < 140 && !is_in_liquid && !playerData.getEffect(MinecraftEffectTypes.speed)) staminaDuration[playerData.id] += 1;
			}else{
				if(staminaDuration[playerData.id] > -1) staminaDuration[playerData.id] -= (playerData.isSneaking) ? 1 : 0.5;
			}
			
			if(stamina_is_cooldown[playerData.id] && vel.y > 0 && !is_in_liquid) playerData.applyKnockback(0, 0, 0, -1);
			if(staminaDuration[playerData.id] >= 140){
				staminaDuration[playerData.id] = 140;
				let speed_value = player_speed.value;
				let speed_decreased = speed_value/1.5;
				let runLoop = system.runInterval(() => {
					player_speed.setCurrent(speed_decreased);
					stamina_is_cooldown[playerData.id] = true;
				});
				let run = system.runInterval(() => {
					system.clearRun(runLoop);
					player_speed.setCurrent(speed_value);
					stamina_is_cooldown[playerData.id] = false;
					system.clearRun(run);
				}, 140);
			}
			if(staminaDuration[playerData.id] >= 0){
				let cooldown_delta = (staminaDuration[playerData.id]+10)/140;
				cooldown_delta = Math.floor(cooldown_delta*8);
				playerData.onScreenDisplay.setActionBar(stamina_cooldown_actionbar[cooldown_delta]);
			}
		}
		
		let time = world.getAbsoluteTime();
		if(rollCooldown[playerData.id] != -1 && rollCooldown[playerData.id] != undefined){
			let cooldown_delta = time - rollCooldown[playerData.id];
			cooldown_delta = Math.floor(cooldown_delta/10);
			if(cooldown_delta <= 5){
				playerData.onScreenDisplay.setActionBar(roll_cooldown_actionbar[cooldown_delta]);
			}else{
				rollCooldown[playerData.id] = -1;
			}
			return;
		}
		
		if(is_in_liquid) return;
		if(playerData.isSneaking){
			if(rollTime[playerData.id] == -1 || rollTime[playerData.id] == undefined){
				rollTime[playerData.id] = time;
			}
		}else{
			if(rollTime[playerData.id] != -1 && rollTime[playerData.id] != undefined){
				if(time - rollTime[playerData.id] < 5){
					let rot = playerData.getRotation();
					let rot_temp = Math.floor((Math.atan2(vel.z, vel.x) * 180 / Math.PI - 90));
					if(rot_temp < -180){
						rot_temp = rot_temp+360;
					}
					let delta_rot = Math.abs(rot_temp - Math.floor((rot.y)))
					delta_rot = Math.min(delta_rot, (360-delta_rot))
					let power = 1;
					let strength = (1-Math.min(Math.abs(vel.y*3), 1))*2.3+0.2;
					let roll_anim = (delta_rot > 120) ? "animation.humanoid.roll_down" : "animation.humanoid.roll_up";
					if(Math.abs(vel.x) + Math.abs(vel.z) > 0.01){
						let vel_up = vel_calc*2;
						playerData.applyKnockback(vel.x, vel.z, strength * ((delta_rot > 120) ? 1.5 : 1), (delta_rot > 120) ? 0.2 : 0.4);
					}else{
						rot.y = (rot.y + 45)* Math.PI/180;
						let velocity = {
							"x": (Math.cos(rot.y)-Math.sin(rot.y))*power,
							"y": 0,
							"z": (Math.sin(rot.y)+Math.cos(rot.y))*power,
						};
						playerData.applyKnockback(velocity.x, velocity.z, strength/2, 0.6);
						roll_anim = "animation.humanoid.roll_up";
					}
					rollCooldown[playerData.id] = world.getAbsoluteTime();
					playerData.playAnimation("animation.humanoid.roll_steady", {
						blendOutTime: 0.3,
						controller: "roll_steady_controller"
					});
					playerData.playAnimation(roll_anim, {
						controller: "roll_controller"
					});
					playerData.playAnimation("animation.humanoid.roll_head", {
						blendOutTime: 1,
						controller: "roll_head_controller"
					});
					if(!playerData.getEffect(MinecraftEffectTypes.jumpBoost)) staminaDuration[playerData.id] += 10;
				}
				rollTime[playerData.id] = -1; 
			}
		}
	}
});

world.events.entityHurt.subscribe( s => {
	let player = s.hurtEntity;
	if(s.damageSource.cause == "magic" || playerHurt[player.id]) return;
	let player_speed = player.getComponent("minecraft:movement");
	let speed_value = player_speed.value;
	let speed_decreased = speed_value/1.5;
	let runLoop = system.runInterval(() => {
		player_speed.setCurrent(speed_decreased);
		playerHurt[player.id] = true;
	});
	let run = system.runInterval(() => {
		system.clearRun(runLoop);
		player_speed.setCurrent(speed_value);
		playerHurt[player.id] = false;
		system.clearRun(run);
	}, 30);
}, {entityTypes: [ "minecraft:player" ]});
